/*
 * parse_index.c
 * Routines to parse the (XML) list index file.
 * bgrot project
 * REQUIRES: libxml2
 * $Fuller: myprograms/bgrot/common/parse/parse_index.c,v 1.13 2003/03/23 14:53:43 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libxml/parser.h>

#include <bgrot.h>
#define PARSE_INDEX
#include <parse_macros.h>
#ifdef BGROTD
#include <bgrotd.h>
#endif

/* Internal prototypes */
int parse_index_list(xmlDocPtr, xmlNodePtr, LINDEX *);

/* Chunks to alloc at once */
#define ALLOC_CHUNK		8


/* parse_index: Parse out the list index file */
LINDEX *
parse_index(const char *indexfile)
{
	xmlDocPtr parsed;
	xmlNodePtr cur;
	LINDEX *toret;
	char *vtmp;

	/* Warm up */
	toret = alloc_lindex();
	if(toret==NULL)
		BAIL("alloc_lindex()");

	/* Parse the file */
	parsed = xmlParseFile(indexfile);
	if(parsed==NULL)
		BAIL("xmlParseFile")

	/* Start picking through the parsed file */
	cur = xmlDocGetRootElement(parsed);
	if(cur==NULL)
		BAIL("xmlDocGetRootElement");

	/* Top level should be <listindex> node */
	if(xmlStrcmp(cur->name, (const xmlChar *) "listindex"))
		BAIL("Should be <listindex>!")
	vtmp = xmlGetProp(cur, (const xmlChar *) "version");
	if(vtmp==NULL)
		BAIL("Can't get version")
	toret->version = (float)atof(vtmp);
	free(vtmp);

	/* Start our big recursion */
	cur = cur->xmlChildrenNode;
	while(cur != NULL)
	{
		if( !(strcmp(cur->name, "list")) )
		{
			if( parse_index_list(parsed, cur, toret) < 0)
				BAIL("parse_index_list failed")
		}
		else if( !(xmlIsBlankNode(cur)) )
		{
			char bgerr[255];
			sprintf(bgerr, "Unknown element '%s'", cur->name);
			BAIL(bgerr);
		}

		cur = cur->next;
	}

	xmlFreeDoc(parsed);

	/* Trim down our memory */
	toret->lists = realloc( toret->lists,
			(toret->lists_used * sizeof(INDEX_ENT *)) );

	return(toret);
}


/* Internal interfaces for parse_index() */
/* Parse one <list> section */
int
parse_index_list(xmlDocPtr parsed, xmlNodePtr cur, LINDEX *toret)
{
	char *tmp;
	INDEX_ENT *thislist;
	INDEX_ENT **tmploc;

	/* Head down the tree */
	cur = cur->xmlChildrenNode;

	/* Check our allocations */
	if(toret->lists_used+1 > toret->lists_alloced)
	{
		toret->lists = realloc( toret->lists,
				((toret->lists_alloced + ALLOC_CHUNK)
					* sizeof(INDEX_ENT *)) );
		if(toret->lists==NULL)
			return(-1);
		tmploc = toret->lists + toret->lists_alloced;
		memset( tmploc, (int)NULL,
				(sizeof(INDEX_ENT *) * ALLOC_CHUNK) );
		toret->lists_alloced += ALLOC_CHUNK;
	}

	/* Alloc our config */
	(toret->lists)[toret->lists_used] = alloc_index_ent();
	if( (toret->lists)[toret->lists_used] == NULL)
		return(-1);
	thislist = (toret->lists)[toret->lists_used];
	toret->lists_used++;

	/* Get the bits of our conf */
	while(cur != NULL)
	{
		if( !(strcmp(cur->name, "id")) )
		{
			tmp = xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
			thislist->id = atoi(tmp);
			free(tmp);
		}
		else if( !(strcmp(cur->name, "title")) )
		{
			thislist->title =
					xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
		}
		else if( !(strcmp(cur->name, "longdesc")) )
		{
			thislist->longdesc =
					xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
		}
		else if( !(strcmp(cur->name, "filename")) )
		{
			thislist->filename =
					xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
		}
		else if( cur->type == XML_COMMENT_NODE )
		{
			/* Comment */
		}
		else if( !(xmlIsBlankNode(cur)) )
		{
			char bgerr[255];
			sprintf(bgerr, "Unknown element '%s'", cur->name);
			BAIL_SUB(bgerr);
		}

		cur = cur->next;
	}

	/* Fin */
	return(0);
}
