/*
 * parse_list.c
 * Routines to parse the (XML) image list file.
 * bgrot project
 * REQUIRES: libxml2
 * $Fuller: myprograms/bgrot/common/parse/parse_list.c,v 1.19 2003/03/23 14:53:43 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libxml/parser.h>

#include <bgrot.h>
#define PARSE_LIST
#include <parse_macros.h>
#ifdef BGROTD
#include <bgrotd.h>
#endif

/* Internal prototypes */
int parse_list_conf(xmlDocPtr, xmlNodePtr, IMGLIST *);
int parse_list_images(xmlDocPtr, xmlNodePtr, IMGLIST *);

/* Chunks to alloc at once */
#define ALLOC_CHUNK		64


/* parse_list: Parse out the image list file */
IMGLIST *
parse_list(const char *ilistfile)
{
	xmlDocPtr parsed;
	xmlNodePtr cur;
	IMGLIST *ilist;
	char *vtmp;

	/* Warm up */
	ilist = alloc_imglist();
	if(ilist==NULL)
		return(NULL);

	/* Parse the file */
	parsed = xmlParseFile(ilistfile);
	if(parsed==NULL)
		BAIL("xmlParseFile")

	/* Start picking through the parsed file */
	cur = xmlDocGetRootElement(parsed);
	if(cur==NULL)
		BAIL("xmlDocGetRootElement");

	/* Top level should be <imglist> node */
	if(xmlStrcmp(cur->name, (const xmlChar *) "imglist"))
		BAIL("Should be <imglist>!")
	vtmp = xmlGetProp(cur, (const xmlChar *) "version");
	if(vtmp==NULL)
		BAIL("Can't get version")
	ilist->version = (float)atof(vtmp);
	free(vtmp);

	/* Start our big recursion */
	cur = cur->xmlChildrenNode;
	while(cur != NULL)
	{
		if( !(strcmp(cur->name, "title")) )
		{
			ilist->title =
					xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
		}
		else if( !(strcmp(cur->name, "longdesc")) )
		{
			ilist->longdesc =
					xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
		}
		else if( !(strcmp(cur->name, "conf")) )
		{
			if( parse_list_conf(parsed, cur, ilist) < 0)
				BAIL("parse_list_conf failed")
		}
		else if( !(strcmp(cur->name, "images")) )
		{
			if( parse_list_images(parsed, cur, ilist) < 0)
				BAIL("parse_list_images failed")
		}
		else if( cur->type == XML_COMMENT_NODE )
		{
			/* Comment */
		}
		else if( !(xmlIsBlankNode(cur)) )
		{
			char bgerr[255];
			sprintf(bgerr, "Unknown element '%s'", cur->name);
			BAIL(bgerr);
		}

		cur = cur->next;
	}

	xmlFreeDoc(parsed);

	/* Trim memory */
	ilist->imgs = realloc( ilist->imgs,
			(ilist->img_used * sizeof(IMGENT *)) );

	return(ilist);
}


/* Internal interfaces for parse_list() */
/* Get the <conf> section slurped in */
int
parse_list_conf(xmlDocPtr parsed, xmlNodePtr cur, IMGLIST *ilist)
{
	char *tmp;

	/* Head down the tree */
	cur = cur->xmlChildrenNode;

	/* Alloc our config */
	ilist->conf = malloc(sizeof(ILCONF));
	if(ilist->conf==NULL)
		BAIL_SUB("Failed to alloc ilist->conf");

	/* Get the bits of our conf */
	while(cur != NULL)
	{
		/* Straight-forward */
		if( !(strcmp(cur->name, "defint")) )
		{
			tmp = xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
			ilist->conf->interval = atol(tmp);
			free(tmp);
			if(ilist->version >= 1.2)
			{
				tmp = xmlGetProp(cur, "defvary");
				ilist->conf->rvary = atoi(tmp);
				free(tmp);
			}
		}
		else if( !(strcmp(cur->name, "defdisp")) )
		{
			ilist->conf->display =
					xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
		}
		else if( cur->type == XML_COMMENT_NODE )
		{
			/* Comment */
		}
		else if( !(xmlIsBlankNode(cur)) )
		{
			char bgerr[255];
			sprintf(bgerr, "Unknown element '%s'", cur->name);
			BAIL_SUB(bgerr);
		}
		

		cur = cur->next;
	}

	/* Fin */
	return(0);
}

/* Get our <images> */
int
parse_list_images(xmlDocPtr parsed, xmlNodePtr cur, IMGLIST *ilist)
{
	IMGENT *imgtmp;
	IMGENT **iltmp;

	/* Head down the tree */
	cur = cur->xmlChildrenNode;

	/* Get the bits of our conf */
	while(cur != NULL)
	{
		if( !(strcmp(cur->name, "image")) )
		{
			/* Check our allocations */
			if(ilist->img_used+1 > ilist->img_alloced)
			{
				ilist->imgs = realloc( ilist->imgs,
						((ilist->img_alloced + ALLOC_CHUNK)
							* sizeof(IMGENT *)) );
				if(ilist->imgs==NULL)
					BAIL_SUB("->imgs realloc()");
				iltmp = ilist->imgs + ilist->img_alloced;
				memset( iltmp, (int)NULL,
						(sizeof(IMGENT *) * ALLOC_CHUNK) );
				ilist->img_alloced += ALLOC_CHUNK;
			}

			(ilist->imgs)[ilist->img_used] = alloc_imgent();
			if( (ilist->imgs)[ilist->img_used] == NULL)
				BAIL_SUB("alloc_imgent");
			imgtmp = (ilist->imgs)[ilist->img_used];
			ilist->img_used++;

			/* Suck the name */
			imgtmp->fname = xmlGetProp(cur, "file");
		}
		else if( cur->type == XML_COMMENT_NODE )
		{
			/* Comment */
		}
		else if( !(xmlIsBlankNode(cur)) )
		{
			char bgerr[255];
			sprintf(bgerr, "Unknown element '%s'", cur->name);
			BAIL_SUB(bgerr);
		}

		cur = cur->next;
	}

	/* Fin */
	return(0);
}
