/*
 * random_img.c
 * 'Normal' randomization routine
 * bgrot project
 * $Fuller: myprograms/bgrot/common/rand/random_img.c,v 1.9 2005/04/12 03:25:31 fullermd Exp $
 */

#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <bgrot.h>
#ifdef BGROTD
#include <bgrotd.h>
#endif


/* Tuning params */
/* Max number of misses before compression */
#define MAX_MISS	5


/*
 * random_img() - Return a random entry from the image list,
 * potentially also compressing the list if we "miss" too many times.
 * And remove it from the list so we don't pick it again.
 */
IMGENT *
random_img(IMGENT ***ilist, int *numimgs)
{
	IMGENT **curlist, **tmp;
	IMGENT *toret;
	int curlist_alloc;
	int nleft;
	int nmiss;
	int dist;
	long randret;
	int done;
	int i, j;

	/* Some initialization */
	curlist = *ilist;
	nmiss = 0;
	done = 0;
	curlist_alloc = *numimgs;

	/* Figure out how many are _really_ left in it */
	nleft=0;
	for(i=0 ; i<curlist_alloc; i++)
	{
		if( curlist[i] != NULL )
			nleft++;
	}

	/*
	 * rand(3) is significantly faster than random(3), and we don't
	 * realy need full crypto-strength randomness.  On the other hand,
	 * random(3) isn't likely to be our bottleneck.  Gopher it.
	 */

	/* Jump into it */
	while(!done)
	{
		/*
		 * If we've passed our 'miss' quota, compact our 'from' array so
		 * we get all hits.
		 *
		 * MAX_MISS tuning may be useful.  The cheaper big memory movings
		 * are relative to random number generation (hint: this ratio is
		 * probably pretty fixed), the lower you want this.  Ideally,
		 * we'd have some self-learning algorithm that would tune itself,
		 * and thus also take into account the fact that compacting is a
		 * hell of a lot more expensive when we've got a thousand items
		 * left than when we've got a dozen items left.  But I'm too
		 * lazy.
		 */
		if(nmiss >= MAX_MISS)
		{
			/* Compact */
			tmp = malloc( (nleft * sizeof(IMGENT *)) );
			if(tmp==NULL)
			{
				/* Error */
				if(curlist!=*ilist)
					free_imgent_pp(curlist, curlist_alloc);
				return(NULL);
			}

			/*
			 * Copy them over.
			 * We're not just schlepping these in.  We're interleaving
			 * them, by putting the first one we find on the end of the
			 * new list, then the second one on the start, then the third
			 * in second-to-last place, the fourth in second place, etc.
			 * The reason we do this is that because of the compression,
			 * the randomization tends to give us severe 'clumping'
			 * toward the end, usually of those that were 'last' on the
			 * list.  This eliminates that, by making sure that the end
			 * and the beginning of the array are different every time,
			 * so pictures end up getting shuffled all through the list,
			 * and THEN getting randomly picked from around it.
			 *
			 * There are probably better ways to do this.  In theory,
			 * this could probably have some really freaky interactions
			 * with the randomization.  In practice, though, this seems
			 * to work pretty well at avoiding notable clumping in the
			 * randomized output, and it's awful cheap to do (heck, even
			 * cheaper than pure linear without optimization, since the
			 * loop is half unrolled  :)
			 */
			j=0;
			dist=0;
			for(i=0 ; i<nleft ; /* nada */)
			{
				/* Put one on the back */
				while( curlist[j] == NULL )
					j++;
				tmp[nleft-dist-1] = curlist[j++];
				if(++i == nleft)
					break;

				/* And one on the front */
				while( curlist[j] == NULL )
					j++;
				tmp[dist] = curlist[j++];
				i++;
				dist++;
			}

			free(curlist);
			curlist = tmp;
			curlist_alloc = nleft;
		}


		randret = (random() % nleft);

		/* Bomb back around if we're a 'miss' */
		if( curlist[randret] == NULL )
		{
			nmiss++;
			continue;
		}


		/*
		 * Else, we got a hit, so get it all ready to return and be done
		 * with the whole mess.
		 */
		toret = alloc_imgent();
		toret->fname = strdup((curlist[randret])->fname);
		free_imgent(curlist[randret]);
		curlist[randret] = NULL;
		done=1;
	}

	/* Smack it all together to return */
	*numimgs = curlist_alloc;
	*ilist = curlist;
	return(toret);
}
